"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.asyncIterSyncMap = asyncIterSyncMap;
function asyncIterSyncMap(source, mapFn) {
    return {
        [Symbol.asyncIterator]: () => {
            let iterator;
            let iterationIdx = 0;
            return {
                next: async () => {
                    iterator ??= source[Symbol.asyncIterator]();
                    const next = await iterator.next();
                    if (next.done) {
                        return next;
                    }
                    const mappedValue = mapFn(next.value, iterationIdx++);
                    return { done: false, value: mappedValue };
                },
                return: async () => {
                    await iterator?.return?.();
                    return { done: true, value: undefined };
                },
            };
        },
    };
}
//# sourceMappingURL=asyncIterSyncMap.js.map