"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useAsyncIterMulti = useAsyncIterMulti;
const react_1 = require("react");
const useSimpleRerender_js_1 = require("../common/hooks/useSimpleRerender.js");
const useRefWithInitialValue_js_1 = require("../common/hooks/useRefWithInitialValue.js");
const isAsyncIter_js_1 = require("../common/isAsyncIter.js");
const asyncIterSyncMap_js_1 = require("../common/asyncIterSyncMap.js");
const ReactAsyncIterable_js_1 = require("../common/ReactAsyncIterable.js");
const iterateAsyncIterWithCallbacks_js_1 = require("../common/iterateAsyncIterWithCallbacks.js");
function useAsyncIterMulti(inputs, opts) {
    const update = (0, useSimpleRerender_js_1.useSimpleRerender)();
    const ref = (0, useRefWithInitialValue_js_1.useRefWithInitialValue)(() => ({
        currDiffCompId: 0,
        prevResults: [],
        activeItersMap: new Map(),
    }));
    const { prevResults, activeItersMap } = ref.current;
    (0, react_1.useEffect)(() => {
        return () => {
            for (const it of activeItersMap.values()) {
                it.destroy();
            }
        };
    }, []);
    const initialValues = opts?.initialValues ?? [];
    const nextDiffCompId = (ref.current.currDiffCompId = ref.current.currDiffCompId === 0 ? 1 : 0);
    let numOfPrevRunItersPreserved = 0;
    const numOfPrevRunIters = activeItersMap.size;
    const nextResults = inputs.map((input, i) => {
        if (!(0, isAsyncIter_js_1.isAsyncIter)(input)) {
            return {
                value: input,
                pendingFirst: false,
                done: false,
                error: undefined,
            };
        }
        const { baseIter, formatFn } = (0, ReactAsyncIterable_js_1.parseReactAsyncIterable)(input);
        const existingIterState = activeItersMap.get(baseIter);
        if (existingIterState) {
            numOfPrevRunItersPreserved++;
            existingIterState.diffCompId = nextDiffCompId;
            existingIterState.formatFn = formatFn;
            return existingIterState.currState;
        }
        const initialValue = prevResults[i] ? prevResults[i].value : initialValues[i]; // TODO: Sure this is the ideal behavior, to use prev iter that happened to be at that position as the value that a new one starts with, even though positions could have been shuffled?
        const formattedIter = (() => {
            let iterationIdx = 0;
            return (0, asyncIterSyncMap_js_1.asyncIterSyncMap)(baseIter, value => newIterState.formatFn(value, iterationIdx++));
        })();
        const destroyFn = (0, iterateAsyncIterWithCallbacks_js_1.iterateAsyncIterWithCallbacks)(formattedIter, initialValue, next => {
            newIterState.currState = { pendingFirst: false, ...next };
            update();
        });
        const newIterState = {
            diffCompId: nextDiffCompId,
            destroy: destroyFn,
            formatFn,
            currState: {
                value: initialValue,
                pendingFirst: true,
                done: false,
                error: undefined,
            },
        };
        activeItersMap.set(baseIter, newIterState);
        return newIterState.currState;
    });
    const numOfPrevRunItersDisappeared = numOfPrevRunIters - numOfPrevRunItersPreserved;
    if (numOfPrevRunItersDisappeared > 0) {
        let i = 0;
        for (const { 0: iter, 1: state } of activeItersMap) {
            if (state.diffCompId !== nextDiffCompId) {
                activeItersMap.delete(iter);
                state.destroy();
                if (++i === numOfPrevRunItersDisappeared) {
                    break;
                }
            }
        }
    }
    return (ref.current.prevResults = nextResults);
}
//# sourceMappingURL=index.js.map