"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IterableChannel = void 0;
const promiseWithResolvers_js_1 = require("../common/promiseWithResolvers.js");
class IterableChannel {
    #isClosed = false;
    #nextIteration = (0, promiseWithResolvers_js_1.promiseWithResolvers)();
    put(value) {
        if (!this.#isClosed) {
            this.#nextIteration.resolve({ done: false, value });
            this.#nextIteration = (0, promiseWithResolvers_js_1.promiseWithResolvers)();
        }
    }
    close() {
        this.#isClosed = true;
        this.#nextIteration.resolve({ done: true, value: undefined });
    }
    iterable = {
        [Symbol.asyncIterator]: () => {
            const whenIteratorClosed = (0, promiseWithResolvers_js_1.promiseWithResolvers)();
            return {
                next: () => {
                    return Promise.race([this.#nextIteration.promise, whenIteratorClosed.promise]);
                },
                return: async () => {
                    whenIteratorClosed.resolve({ done: true, value: undefined });
                    return { done: true, value: undefined };
                },
            };
        },
    };
}
exports.IterableChannel = IterableChannel;
//# sourceMappingURL=IterableChannel.js.map