import { useEffect } from 'react';
import { useSimpleRerender } from '../common/hooks/useSimpleRerender.js';
import { useRefWithInitialValue } from '../common/hooks/useRefWithInitialValue.js';
import { isAsyncIter } from '../common/isAsyncIter.js';
import { asyncIterSyncMap } from '../common/asyncIterSyncMap.js';
import { parseReactAsyncIterable } from '../common/ReactAsyncIterable.js';
import { iterateAsyncIterWithCallbacks } from '../common/iterateAsyncIterWithCallbacks.js';
export { useAsyncIterMulti };
function useAsyncIterMulti(inputs, opts) {
    const update = useSimpleRerender();
    const ref = useRefWithInitialValue(() => ({
        currDiffCompId: 0,
        prevResults: [],
        activeItersMap: new Map(),
    }));
    const { prevResults, activeItersMap } = ref.current;
    useEffect(() => {
        return () => {
            for (const it of activeItersMap.values()) {
                it.destroy();
            }
        };
    }, []);
    const initialValues = opts?.initialValues ?? [];
    const nextDiffCompId = (ref.current.currDiffCompId = ref.current.currDiffCompId === 0 ? 1 : 0);
    let numOfPrevRunItersPreserved = 0;
    const numOfPrevRunIters = activeItersMap.size;
    const nextResults = inputs.map((input, i) => {
        if (!isAsyncIter(input)) {
            return {
                value: input,
                pendingFirst: false,
                done: false,
                error: undefined,
            };
        }
        const { baseIter, formatFn } = parseReactAsyncIterable(input);
        const existingIterState = activeItersMap.get(baseIter);
        if (existingIterState) {
            numOfPrevRunItersPreserved++;
            existingIterState.diffCompId = nextDiffCompId;
            existingIterState.formatFn = formatFn;
            return existingIterState.currState;
        }
        const initialValue = prevResults[i] ? prevResults[i].value : initialValues[i]; // TODO: Sure this is the ideal behavior, to use prev iter that happened to be at that position as the value that a new one starts with, even though positions could have been shuffled?
        const formattedIter = (() => {
            let iterationIdx = 0;
            return asyncIterSyncMap(baseIter, value => newIterState.formatFn(value, iterationIdx++));
        })();
        const destroyFn = iterateAsyncIterWithCallbacks(formattedIter, initialValue, next => {
            newIterState.currState = { pendingFirst: false, ...next };
            update();
        });
        const newIterState = {
            diffCompId: nextDiffCompId,
            destroy: destroyFn,
            formatFn,
            currState: {
                value: initialValue,
                pendingFirst: true,
                done: false,
                error: undefined,
            },
        };
        activeItersMap.set(baseIter, newIterState);
        return newIterState.currState;
    });
    const numOfPrevRunItersDisappeared = numOfPrevRunIters - numOfPrevRunItersPreserved;
    if (numOfPrevRunItersDisappeared > 0) {
        let i = 0;
        for (const { 0: iter, 1: state } of activeItersMap) {
            if (state.diffCompId !== nextDiffCompId) {
                activeItersMap.delete(iter);
                state.destroy();
                if (++i === numOfPrevRunItersDisappeared) {
                    break;
                }
            }
        }
    }
    return (ref.current.prevResults = nextResults);
}
//# sourceMappingURL=index.js.map