import { promiseWithResolvers } from '../common/promiseWithResolvers.js';
export { IterableChannel };
class IterableChannel {
    #isClosed = false;
    #nextIteration = promiseWithResolvers();
    put(value) {
        if (!this.#isClosed) {
            this.#nextIteration.resolve({ done: false, value });
            this.#nextIteration = promiseWithResolvers();
        }
    }
    close() {
        this.#isClosed = true;
        this.#nextIteration.resolve({ done: true, value: undefined });
    }
    iterable = {
        [Symbol.asyncIterator]: () => {
            const whenIteratorClosed = promiseWithResolvers();
            return {
                next: () => {
                    return Promise.race([this.#nextIteration.promise, whenIteratorClosed.promise]);
                },
                return: async () => {
                    whenIteratorClosed.resolve({ done: true, value: undefined });
                    return { done: true, value: undefined };
                },
            };
        },
    };
}
//# sourceMappingURL=IterableChannel.js.map